/* @(#) $Revision: 1.1.109.2 $ */

#ifndef _FMT_INCLUDED
#define _FMT_INCLUDED

/* BEGIN_IMS fmt.h *
 ****************************************************************************
 ****
 ****	fmt.h 
 ****
 ****************************************************************************
 *
 * Description   
 *
 *	External header file for the trace/log formatter.
 *	For use by the upper and lower users of these functions,
 *	specifically, the subsystem formatters,the subsystem get_options
 *	functions, and the user interface for the formatter, netfmt.
 *
 * NOTE
 *	Users of the formatter will need to include the global
 *	include file for trace/log. It is probably named ntl.h.
 *
 *******************************************************************************
 * END_IMS fmt.h */


/*******************************************************************
* types defined in the ES
********************************************************************/

typedef  struct  
{
   unsigned  trace_log_bit	: 1;	/* set for log */
   unsigned  options_only_bit   : 1;	/* parse options only, no formatting */
   unsigned  console_logging    : 1;
   unsigned  reserved		: 29;

} tl_fmt_flag_type;

typedef  struct  
{
   unsigned  trace_log_bit      : 1;
   unsigned  reserved		: 31;
} get_opt_flag_type;


typedef  struct  
{
   unsigned  verbosity_bit      : 1;
   unsigned  console_logging    : 1;
   unsigned  reserved		: 30;
} ss_N_fmt_flag_type;

typedef struct 
{
   int     fd;          /*file descriptor*/
   int     result;
} fd_result;

typedef struct 
{
   int     fd;          /*file descriptor*/
   int     result;
} fp_result;		/* fp was used in an earlier version of the ES */

typedef struct 
{
   char    *user_acct;
   int     result;
} user_acct_result;

typedef  struct
{
   unsigned console_logging     : 1;
   unsigned reserved            : 31;
} hdr_fmt_flag_type;

typedef   int       raw_fmt_flag_type;
typedef   int       fmt_wrt_flag_type;


/*******************************************************************
 * -tail,-Follow option for netfmt 
 *******************************************************************/

#define  MAX_TAIL_SIZE 1000
#define  DEFAULT_TAIL_SIZE 50
#define  FOLLOW_OPTION   0x10000000
#define  TAIL_OPTION     0x20000000
#define  STAT_OPTION     0x40000000
#define  TAIL_SIZE_MASK  0x0000ffff
#define  FMT_OPTION_MASK 0xffff0000

/*******************************************************************
 * Misc. external defines
 *******************************************************************/

#define  NO_PASS   0
#define  PASS      1

/*******************************************************************
* error constants
********************************************************************/

#define  FMTERR_INV_FLAGS        1
#define  FMTERR_OUT_CREATE       2
#define  FMTERR_ACCESS_IN        3
#define  FMTERR_SYS_ERROR        4
#define  FMTERR_NO_MEM           5
#define  FMTERR_INV_OPT_FP       6
#define  FMTERR_INV_OPT_FD       7
#define  FMTERR_INV_MC_FP        8
#define  FMTERR_INV_MC_FD        9
#define  FMTERR_INV_BIN_MP       10
#define  FMTERR_INV_TL_MSG       11

/* for the library procedures   */

#define  FMTERR_INV_HDR_PTR      12
#define  FMTERR_INV_HDR          13
#define  FMTERR_INV_OUT_FP       14
#define  FMTERR_INV_OUT_FD       15
#define  FMTERR_INV_NBYTES       16
#define  FMTERR_INV_DATA_PTR     17
#define  FMTERR_FORMAT_WRITE     18
#define  FMTERR_INV_L_STR        19

#endif
