/*
 * @(#)pde.h: $Revision: 1.24.61.13 $ $Date: 93/02/24 12:32:53 $
 * $Locker:  $
 */

#ifndef	_MACHINE_PDE_INCLUDED
#define	_MACHINE_PDE_INCLUDED

#ifdef _KERNEL_BUILD
#  include "../machine/param.h"
#  include "../h/vmmac.h"
#  ifndef LOCORE
#    include "../h/spinlock.h"
#  endif
#else  /* !_KERNEL_BUILD */
#  include <machine/param.h>
#  include <sys/vmmac.h>
#  include <sys/spinlock.h>
#endif	/* _KERNEL_BUILD */

#define MP_SPARSE_PDIR

#ifndef LOCORE

#define PDE_VPAGE PGNDXWDTH
#define PDE_VOFF PGSHIFT

#define PDE_PPAGE PGNDXWDTH

/* Convert virtual page to the fifteen-bit format used within the pde */
#define VPAGE_TO_PDE_VPAGE(vpage) ((u_int)(((u_int)vpage >> 5) & 0x7FFF))

/* Notes on pde structure:
 * Separating spaces indicate boundaries between words within the
 * structure.  There is a field in the comment at the end of each field
 * that indicates that is flagged 'T', 'H', both, or neither.  An 'H' indicates
 * that the field affects the hardware and it's placement is critical.
 * A 'T' indicates that certain implementations require placement of the field
 * in the indicated position.  Those marked neither 'T' nor 'H' are simply
 * software conventions.
 */

/* SCF - "Stingy cache flush." */

struct hpde {
 
  /* Word 0 */
    volatile
	unsigned int	pde_valid_e    :1;	/* Valid bit                T */
	unsigned int	pde_page_e     :15;	/* V. page - high 15 bits   T */
	unsigned int	pde_space_e    :16;	/* Virtual space number     T */

  /* Word 1 */
    volatile
	unsigned int	pde_ref_e      :1;	/* Reference bit            T */
    volatile
	unsigned int	pde_accessed_e :1;	/* SCF flag-may be in cache   */
	unsigned int	pde_rtrap_e    :1;	/* Data ref trap enable bit  H*/
    volatile
	unsigned int	pde_dirty_e    :1;	/* Dirty bit                 H*/
	unsigned int	pde_dbrk_e     :1;	/* Data break                H*/
	unsigned int	pde_ar_e       :7;	/* Access rights             H*/
	unsigned int	pde_uncache_e  :1;	/* page is non-cacheable    TH*/
	unsigned int		       :1;
	unsigned int	pde_shadow_e   :1;	/* Shadow                     */
    volatile
	unsigned int	pde_executed_e :1;	/* SCF flag - in I-cache      */
	unsigned int	pde_protid_e   :15;	/* Protection ID             H*/
    volatile
	unsigned int	pde_modified_e :1;	/* VM high-lvl page-mod bit   */

  /* Word 2 */
    volatile
	unsigned int	pde_uip_e   :1;		/* Update in progress.        */
	unsigned int                :31-5-PDE_PPAGE;
	unsigned int	pde_phys_e  :PDE_PPAGE; /* Physical page            TH*/
	unsigned int                :5;

  /* Word 3 */
	struct hpde    *pde_next;		/* Next entry               T */

  /* Word 4 */
	unsigned int	pde_valid_o    :1;	/* Valid bit                T */
	unsigned int	pde_page_o     :15;	/* V. page - high 15 bits   T */
	unsigned int	pde_space_o    :16;	/* Virtual space number     T */

  /* Word 5 */
    volatile
	unsigned int	pde_ref_o      :1;	/* Reference bit            T */
    volatile
	unsigned int	pde_accessed_o :1;	/* SCF flag-may be in cache   */
	unsigned int	pde_rtrap_o    :1;	/* Data ref trap enable bit  H*/
    volatile
	unsigned int	pde_dirty_o    :1;	/* Dirty bit                 H*/
	unsigned int	pde_dbrk_o     :1;	/* Data break                H*/
	unsigned int	pde_ar_o       :7;	/* Access rights             H*/
	unsigned int	pde_uncache_o  :1;	/* page is non-cacheable    TH*/
	unsigned int		       :1;
	unsigned int	pde_shadow_o   :1;	/* Shadow                     */
    volatile
	unsigned int	pde_executed_o :1;	/* SCF flag - in I-cache      */
	unsigned int	pde_protid_o   :15;	/* Protection ID             H*/
    volatile
	unsigned int	pde_modified_o :1;	/* VM high-lvl page-mod bit   */

  /* Word 6 */
    volatile
	unsigned int	pde_uip_o   :1;		/* Update in progress.        */
	unsigned int	            :31-5-PDE_PPAGE;
	unsigned int	pde_phys_o  :PDE_PPAGE;	/* Physical page            TH*/
	unsigned int	            :5;

  /* Word 7 */
	unsigned int	pde_block_mapped :1;	/* Block mapping flag         */
	unsigned int	pde_ref_trickle_e:1;    /* Trickle-up bit for ref's.  */
	unsigned int	pde_ref_trickle_o:1;    /* Trickle-up bit for ref's.  */
	unsigned int	pde_alias        :27;	/* Virtual alias field        */
	unsigned int	pde_os           :2;	/* Usage bits: entry free?    */
};

typedef struct hpde * 	hpde_handle_t;	/* just a handle, not to be deref'd */

	/* Word oriented hpde structure, for fast moves */
struct whpde {
 
	unsigned int	wpde_vaddr_e;   /* Virtual space, page, and valid bit */
    volatile
	unsigned int	wpde_acword_e;	
    volatile
	unsigned int	wpde_phys_e;	
	unsigned int	wpde_next;	
	unsigned int	wpde_vaddr_o;   /* Virtual space, page, and valid bit */
    volatile
	unsigned int	wpde_acword_o;	
    volatile
	unsigned int	wpde_phys_o;	
	unsigned int	wpde_os;	
};


/*
 * PDE_LOG2 - log2(size of a pdir entry).  Used by handlers
 * to index into pdir array. Sparse pdir doesn't index into hash table in
 * handlers. Include it since people may expect it to be here.
 */
#define PDE_LOG2	5	/* LOG2(sizeof(struct pde)) */

#ifdef _KERNEL
extern struct hpde *base_pdir;	/* Pointer to lowest valid pde address. */
extern struct hpde *max_pdir;	/* Pointer to highest valid pde address+1. */
extern struct hpde *pdir;	/* Pointer to non-hashed pde entries. */
extern struct hpde *htbl;	/* Pointer to hash table */
extern struct hpde *htbl_end;	/* Pointer to end of hash table */
extern hpde_handle_t vtopde();
extern struct hpde **pgtopde_table;
extern u_int scaled_npdir;
extern unsigned int npdir;	/* Number of (positive) pdir entries */
extern unsigned int nhtbl;	/* Number of htlb entries */
extern	int	io_protid;	/* Next io protid to allocate */


/*
 * These constants are used when enabling the HW walker for a PCX-T
 * system. They tell the enabling function if we are under powerfail
 * recovery.
 *
 * In the future, all of these processor dependent constants should
 * probably have their own header file.
 */
#define HWTLB_PFAIL_ENABLE	1
#define HWTLB_NORMAL_ENABLE	0

/*
 * Definition of the lock that guards PDIR access.
 */

extern lock_t *pd_lock;

#define	PDMODGET(PFN)	((hdl_getbits(PFN) & VPG_MOD) == VPG_MOD)
#define	PDREFGET(PFN)	((hdl_getbits(PFN) & VPG_REF) == VPG_REF)

#endif /* _KERNEL */

#define PDE_INVALID 0x0000FFFF

#define	NUMPDIR			(NBPG/sizeof(struct hpde))
#define HPDIRFREE	0xffffffff

/* MAX_SPACE_IDs must be a power of 2. */
#define MAX_SPACE_IDs	32768	/* Valid space IDs 0-32767 */

#define INVALID_SPACE1 (0xFFFF)
#define INVALID_SPACE2 (0xFFFFFFFF)

/* 
 * When 1/2 a hash bucket is empty it should contain a ref bit of 0 and a 
 * physical page of 0
 */
#define HPHYSINV	0x0
#define HREFINV		0x0
#define HOSINV		0x0

/*
 * The following hash function produces a 32 bit index, (hopefully)
 * more than we will ever need.
 */
#define pdirhash(sid, sof)	((sid << 4 ) ^ ((u_int) (sof) >> (PGSHIFT+1)))

#endif /* LOCORE */

/* Constants related to field positions within the PDE structure, and used
 * by the low-level code for greater readability.
 */
#define	PDE_VADDR	0
#define	PDE_PROT	4
#define	PDE_PPN 	8
#define	PDE_NEXT	12
#define	PDE_UIP_BYTE	8
#define	PDE_UIP_BIT	24
#define	PDE_UIP_WBIT	0
#define PDE_OS			28
#define PDE_BLOCK_MAPPED	0


#define PA83_ODD_PAGE	20

/* HSPOS (Hash Shift POSition) is the low-order bit position of the space-ID
 * in the selected hash algorithm.
 */
#define	HSPOS   	26

/* HSWDTH (Hash Shift WiDTH) is the number of bits retained from the space-ID
 * in the selected hash algorithm.
 */
#define	HSWDTH 		16

/*
 * Access rights are now given out on a region basis, thus all the pages
 * in a region have the same access.
 */
#define	PDE_AR_KR	0x00
#define	PDE_AR_KRW	0x10
#define	PDE_AR_KRX	0x20
#define	PDE_AR_KRWX	0x30

#define	PDE_AR_UR	0x0f
#define	PDE_AR_URW	0x1f
#define	PDE_AR_URX	0x2f
#define	PDE_AR_URWX	0x3f
#define PDE_AR_UX	0x7f

#define	PDE_AR_URKW	0x1c
#define	PDE_AR_URWKW	0x1f
#define	PDE_AR_URXKW	0x3c
#define	PDE_AR_URWXKW	0x3f
#define PDE_AR_INVALID	0xff

#define PDE_HASEXECUTE	0x20
#define PDE_HASWRITE	0x10
#define PDE_AR_GX	0x4c

#define PDE_REF_POS        0
#define PDE_ACCESSED_POS   1
#define PDE_RTRAP_POS      2
#define PDE_DIRTY_POS      3
#define PDE_DBRK_POS       4
#define PDE_UNCACHE_POS   12
#define PDE_EXECUTED_POS  15
#define PDE_MODIFIED_POS  31

#define PDE_REF_BIT       0x80000000
#define PDE_ACCESSED_BIT  0x40000000
#define PDE_RTRAP_BIT     0x20000000
#define PDE_DIRTY_BIT     0x10000000
#define PDE_DBRK_BIT      0x08000000
#define PDE_UNCACHE_BIT   0x00080000
#define PDE_EXECUTED_BIT  0x00010000
#define PDE_MODIFIED_BIT  0x00000001

#define PDE_REF_TRICKLE_E_POS  1
#define PDE_REF_TRICKLE_O_POS  2

/* Also keep old ones for now  XXX */

#define PDE_AR_URKW	0x1c
#define PDE_AR_KXR	0x20
#define PDE_AR_KXRW	0x30
#define PDE_AR_KXUX	0x2c
#define PDE_AR_KX	0x20
#define	PDE_AR_UXRW	0x3f
#define	PDE_AR_UXR	0x2f
#define PDE_AR_CW	PDE_AR_UXR
#define PDE_AR_URXKW	0x3c


#define spreg(x)	(((int)x>>30)&0x3)

#define NIOPDE	2*256	/* number of pde's for I/O area of PDIR */
#define SIOPDE	2*1	/* negative start of valid pde's for I/O area of PDIR */

/* Access rights equates as defined by Azure kernel */
#define PDE_AR_GATE	PDE_AR_GX
#define PDE_AR_URXKR	PDE_AR_UXR
#define PDE_AR_NOACC	0x73
#define PDE_AR_TRANSIT	0x72  /* No Access, page being pushed out. */


#define PIDMASK		0xfffe
#define PROTID_MASK	0xffff
#define PROTID_RESTRICT	0x80000000
#define PROTID_REST_POS	0
#define PROTID_SUSPEND	0x40000000
#define PROTID_SLEEPERS	0x20000000
#define PROTID_DELETED	0x10000000

#define PDE_SHADOW_OFF 1
#define PDE_SHADOW_BIT 0x20000
#define PDE_SHADOW_BOFFSET NBPG
#define PDE_ODD_BIT NBPG

/*
 * Macro for converting between HI page frame  numbers and HD
 * addresses.
 */
#define HPPA_NBPG NBPG
#define HPPA_ptob(PGS) ((PGS) * HPPA_NBPG)
#define pfn_to_ra(PFN) (HPPA_ptob((PFN)))

#endif	/* ! _PDE_INCLUDED */
