/*
 * @(#)space.h: $Revision: 1.88.61.23 $ $Date: 93/05/28 12:05:49 $
 * $Locker:  $
 *
 */

#ifndef _MACHINE_SPACE_INCLUDED /* allows multiple inclusion */
#define _MACHINE_SPACE_INCLUDED

/*
 * This file should NOT be included by user programs.
 */
#ifdef _KERNEL

#ifndef _SYS_STDSYMS_INCLUDED
#    include "../h/stdsyms.h"
#endif   /* _SYS_STDSYMS_INCLUDED  */

#include "../h/param.h"
#include "../h/sysmacros.h"
#include "../h/buf.h"
#include "../h/dir.h"
#include "../h/user.h"
#include "../h/proc.h"
#include "../h/kern_sem.h"
#undef u
#include "../h/conf.h"
#include "../h/tty.h"
#include "../machine/cpu.h"
#include "../sio/llio.h"
#include "../sio/iotree.h"
#include "../h/utsname.h"
#include "../sio/dac.h"
#ifndef _WSIO	/* weed out the case of s700 */
#include "config.h"		/* configurable definitions */
#endif	/* !_WSIO */
#include "../dux/sitemap.h"
#include "../h/file.h"
#include "../h/opt.h"		/* stubs for configured out routines */
#include "../machine/pfail.h"
#include "../h/dnlc.h"
#include "../h/sleep.h"

#include "../dux/rmswap.h"
#ifdef DSKLESS
#include "../dux/dm.h"
#include "../dux/protocol.h"
#include "../dux/nsp.h"
#endif /* DSKLESS */
#include "../h/devices.h"

#ifdef MIRROR
#include "../sio/mirror2.h"
#endif /* MIRROR */
/*
 *	Begin System Configurable parameters.
 *
 *	System configurable parameters
 *	Basically, what we are doing here is setting the value of
 *		global variables which depend on #defines set up
 *		in config.h by uxgen. This allows the user to set
 *		their values in an uxgen script.
 *
 *	The rules for inclusion in this part of the file are as follows:
 *		1.  Must be a parameter the user can configure
 *			at uxgen time.
 *		2.  No file except conf.c can depend on these
 *			values changing.
 */

/*
 * Tunable parameter iomemsize specifies the amount of memory (in bytes)
 * to reserve for emergency I/O services memory used by io_get_mem().
 * io_mem_buf[] must be equivalently mapped.
 */
#define NUM_IO_MEM_PAGES howmany(IOMEMSIZE, NBPG)

#ifndef _WSIO
int	iomemsize = IOMEMSIZE;
int	num_io_mem_pages = NUM_IO_MEM_PAGES;
char	io_mem_buf[(NUM_IO_MEM_PAGES * NBPG) + NBPG-1];
#endif

/* Accounting parameters */

int acctresume = ACCTRESUME;

int acctsuspend = ACCTSUSPEND;

/*
 * Maximum size of program text, data and stack regions.
 */

long maxdsiz = MAXDSIZ/NBPG; /* unit: page size */
long maxssiz = MAXSSIZ/NBPG; /* unit: page size */
long maxtsiz = MAXTSIZ/NBPG; /* unit: page size */


/*
 * The following should be uncommented when the 7.0 uxgen becomes available.
 */
/*
int dmmax = DMMAX;
int dmtext = DMTEXT;
#ifdef SHMEM
int dmshm  = DMSHM;
#endif
int dmmin = DMMIN;
*/


/*
 * Maximum number of processes per user
 */

int maxuprc = MAXUPRC;

/*
 *	Buffer headers for block I/O system
 */
int	nbuf = NBUF;		/* number of buffer headers */

#ifndef BUFPAGES
#define BUFPAGES 0
#endif /* BUFPAGES */
int	bufpages = BUFPAGES;	/* number of memory pages in the buffer pool */

/*
 *	Number of proc table entries
 */
int	nproc = NPROC;

/*
 *	Hash table of open devices.
 */
dtaddr_t devhash[DEVHSZ];

/*
 *	If certain networking stuff is defined, we will need more
 *	of certain resources
 */
#define NETSLOP 20			/* for all the lousy servers*/

#ifndef RAMFS
char        *ramfsbegin = NULL;
unsigned int ramfssize = 0;
#endif

#ifdef NOSWAP
#define NOSWAP 1
#else
#define NOSWAP 0
#endif

int nopaging = NOSWAP;
int noswapping = NOSWAP;
int noswap = NOSWAP;

/*
 *  Number of text entries
 */

int	ntext = NTEXT;

/*
 *	Number of Inodes, Number of file table entries
 */
int	ninode = NINODE;

int	nfile = NFILE + FILE_PAD;
int	file_pad = FILE_PAD;
/*
 * maxfiles is the system default soft limit for the maximum number of
 * open files per process.  maxfiles defaults to 60 if not configured.
 * maxfiles_lim is the system default hard limit for the maximum number of
 * open files per process.  maxfiles_lim defaults to 1024 if not configured.
 */

int	maxfiles = MAXFILES;
int	maxfiles_lim = MAXFILES_LIM;


/*
 *	Size of Directory Name Lookup Cache
 */
int	ncsize = NINODE;
struct	ncache ncache[NINODE];

/*
 *	Number of callout structures
 */
int	ncallout = NCALLOUT;

/*
 * Flag to enable or disable powerfail.
 */
int	pfail_enabled = PFAIL_ENABLED;

/*
 *	Number of IO services timers
 */
struct timer_map timer_list[NCALLOUT];
struct timer_map *free_timer_list;

/*
 *	File locking.
 *	The number of files which can be locked.
 */
#include "../h/vnode.h"
#include "../h/vfs.h"
#include "../ufs/inode.h"
int nflocks = NFLOCKS;
struct locklist locklist[NFLOCKS];	/* The lock table itself */

/*
 *	Process locking.
 *	Here is defined the amount of memory in the system which
 *	can never be locked.  If it is zero here, it is initialized
 *	at boot time in init_main (to some non-zero amount).
 */
int	unlockable_mem = UNLOCKABLE_MEM;/* Amount reserved from locking */

int timeslice = TIMESLICE;

/*
 *	Pseudo-drivers
 *	Set max ptys allowed
 */
#include "../h/pty.h"
#include "../h/nvs.h"
long npty = NPTY;

struct	tty		pt_tty[NPTY];
struct	tty		*pt_line[NPTY];
struct	pty_info	pty_info[NPTY];
struct	nvsj		nvsj[NPTY];

#if MESG==1
#include	"../h/ipc.h"
#include	"../h/msg.h"

struct ipcmap	msgmap[MSGMAP];
struct msqid_ds msgque[MSGMNI];
struct msg	msgh[MSGTQL];
struct msginfo	msginfo = {
	MSGMAP,
	MSGMAX,
	MSGMNB,
	MSGMNI,
	MSGSSZ,
	MSGTQL,
	MSGSEG
};
#endif

/*
 * The following is for the semaphore subsystem (if configured)
 */
#if SEMA==1
#	ifndef IPC_ALLOC
#	include "../h/ipc.h"
#	endif
#include	"../h/sem.h"

/*
 * SEMMSL, max number of semaphores per sem id, is defined in sem.h
 * SEMOPM, maxi number of operations per semop system call, is defined
 *	in sem.h
 */

struct semid_ds sema[SEMMNI];
struct sem	sem[SEMMNS];
struct ipcmap	semmap[SEMMAP];
struct	sem_undo	*sem_undo[NPROC];

#define SEMUSZ	(sizeof(struct sem_undo)+sizeof(struct undo)*SEMUME)

int	semu[((SEMUSZ*SEMMNU)+NBPW-1)/NBPW];

struct	seminfo seminfo = {
	SEMMAP,
	SEMMNI,
	SEMMNS,
	SEMMNU,
	SEMMSL,
	SEMOPM,
	SEMUME,
	SEMUSZ,
	SEMVMX,
	SEMAEM
};
#endif

/*
 * The following is for the Shared memory subsystem (if configured in)
 */
#	ifndef	IPC_ALLOC
#	include "../h/ipc.h"
#	endif
#include	"../h/shm.h"

#ifdef SHMEM

/*
 * Minimum amount of shared memory for one segment (not tuneable)
 */
#define SHMMIN	1

/*
 * No effect, included only for Sys V comatibility
 */
#define SHMBRK	16

/*
 * No effect, included only for Sys V compatibility
 */
#define SHMALL	0x600

int shmmni = SHMMNI;	/* used by vm_machdep */
#define NSHM_FREE_SEGS 0  /* they no longer exist. this keeps shminfo struct */
			  /* the same size. */

struct	shmid_ds	shmem[SHMMNI];
struct	shminfo shminfo = {
	SHMMAX,
	SHMMIN,
	SHMMNI,
	SHMSEG,
};
#else
int     shmmni = 1;     /* used by vm_machdep */
struct  shmid_ds shmem[1];
#define SHMSEG 0
#endif

/*
 * Software TLB
 */
int	nstlbe = NSTLBE;

/*
 * Public Keys on shared libraries
 */
int   public_shlibs = PUBLIC_SHLIBS;

/*
 * End System configurable parameters
 */

int	hz = HZ;
int	tick = 1000000 / HZ;
struct	timezone tz = { TIMEZONE, DST };
int	nclist = 100 + 16 * MAXUSERS;
int	nport = NPROC / 2;

#ifdef DSKLESS
short rootlink[3] = { 0xffff, 0xffff, 0xffff };
int lanselectcode = -1;
int num_cnodes           = NUM_CNODES;
int dskless_fsbufs       = (DSKLESS_FSBUFS > MAX_SERVING_ARRAY) ? MAX_SERVING_ARRAY : DSKLESS_FSBUFS;
#else
int dskless_fsbufs	 = 0;
#endif /* DSKLESS */
int reboot_option        = REBOOT_OPTION;
int dskless_node         = DSKLESS_NODE;
int server_node          = SERVER_NODE;
int brmtdev =  6;        /* index of rdu driver in bdevsw[] */
int crmtdev = 45;       /* index of rdu driver in cdevsw[] */


#ifdef DSKLESS
/*
** Size the using/serving arrays. USING_ARRAY_SIZE and SERVING_ARRAY_SIZE
** are configurable parameters.
*/
int using_array_size = USING_ARRAY_SIZE;
struct using_entry using_array[ USING_ARRAY_SIZE ];

int serving_array_size = (SERVING_ARRAY_SIZE > MAX_SERVING_ARRAY) ? MAX_SERVING_ARRAY : SERVING_ARRAY_SIZE;
struct serving_entry serving_array[ (SERVING_ARRAY_SIZE > MAX_SERVING_ARRAY) ? MAX_SERVING_ARRAY : SERVING_ARRAY_SIZE];

/*
** Define timeout periods for selftest and crash detection.  SELFTEST_PERIOD,
** SEND_ALIVE_PERIOD and CHECK_ALIVE_PERIOD are configurable parameters.
*/

/* If selftest period is 0 then no selftest, otherwise lowerbound of 90 secs */
int selftest_period = ((SELFTEST_PERIOD == 0) ? SELFTEST_PERIOD : ((SELFTEST_PERIOD < 90) ? 90 : SELFTEST_PERIOD));

int check_alive_period = CHECK_ALIVE_PERIOD;
int retry_alive_period = RETRY_ALIVE_PERIOD;

int ngcsp = NGCSP;
int ncsp = NGCSP + 1;			/* always one for limited CSP */
struct nsp nsp[NGCSP+1];		/* always one for limited CSP */
struct nsp *nspNCSP = &nsp[NGCSP+1];

#endif /* DSKLESS */

/*
 * The following is for the BFA subsystem (if configured)
 */
#ifdef	BFA
#include "../h/bfa.h"
#define BFANFUNCTS	1400	/* uxgen should generate this */
#define BFAMEMSIZE	(BFANFUNCTS*sizeof(struct functrecord))
char bfamem[BFAMEMSIZE];
int bfamemsize = BFAMEMSIZE;
#endif

#ifdef UIPC    /* UIPC is the umbrella subsystem for networking */

/*
 * Networking
 */

#include "../h/mbuf.h"
#define PRUREQUESTS
#include "../h/protosw.h"
#include "../h/socket.h"

#ifdef INET
#include "../net/if.h"
#include "../net/route.h"
#include "../net/raw_cb.h"
#include "../netinet/in.h"
#include "../netinet/if_ether.h"
#include "../h/mib.h"
#include "../netinet/mib_kern.h"
#include "../net/if_ni.h"
/* NI */
int ni_max = NNI;
struct ni_cb ni_cb[NNI];

/*
 * Internet Domain
 */
#define TCPSTATES
#include "../netinet/tcp_fsm.h"

struct ifqueue ipintrq;

/*
 * (X)NS Domain
 */
struct ifqueue nsintrq;

#endif /* INET */
#endif /* UIPC */

/*
 * Common Network Layer
 */
int	netisr_priority = NETISR_PRIORITY;
int     netmemmax       = NETMEMMAX;


/*
 * Disc statistics, used by iostat(util), vmstat(util) and monitor(util).
 */

#ifndef _WSIO /* s800 only, not s700 */
#ifdef NUM_DISC0_L1
int	dk_ndrive = NUM_DISC0_L1;
char	dk_busy[NUM_DISC0_L1] = {0};	/* has to stay in DATA */
long	dk_time[NUM_DISC0_L1];
long	dk_seek[NUM_DISC0_L1];
long	dk_xfer[NUM_DISC0_L1];
long	dk_wds[NUM_DISC0_L1];
float	dk_mspw[NUM_DISC0_L1];
long	dk_time_snap[NUM_DISC0_L1];   /* xxx */
struct timeval dk_resp[NUM_DISC0_L1]; /* xxx */
#else
int	dk_ndrive = 0;
char	dk_busy[1] = {0};	/* has to stay in DATA */
long	dk_time[1];
long	dk_seek[1];
long	dk_xfer[1];
long	dk_wds[1];
float	dk_mspw[1];
long	dk_time_snap[1];
struct timeval dk_resp[1];
#endif
#endif /* not WSIO */

/* Measurement hooks for disc1.c */
#define MAX_DISK_DEVICES   256
int	dk_ndrive1 = 0;
int	dk_busy1[MAX_DISK_DEVICES] = {0};
long	dk_time1[MAX_DISK_DEVICES];
long	dk_time_snap1[MAX_DISK_DEVICES];
long	dk_seek1[MAX_DISK_DEVICES];
long	dk_xfer1[MAX_DISK_DEVICES];
long	dk_wds1[MAX_DISK_DEVICES];
float	dk_mspw1[MAX_DISK_DEVICES];
struct timeval dk_resp1[MAX_DISK_DEVICES];

#ifdef SADP
int	dk_cyl_index1[MAX_DISK_DEVICES] = {0};
int	*dk_cyl1[MAX_DISK_DEVICES];
#endif

/* Measurement hooks for disc2.c */
int	dk_ndrive2 = 0;
int	dk_busy2[MAX_DISK_DEVICES * 2] = {0};
long	dk_time2[MAX_DISK_DEVICES];
long	dk_time_snap2[MAX_DISK_DEVICES];
long	dk_seek2[MAX_DISK_DEVICES];
long	dk_xfer2[MAX_DISK_DEVICES];
long	dk_wds2[MAX_DISK_DEVICES];
float	dk_mspw2[MAX_DISK_DEVICES];
struct timeval dk_resp2[MAX_DISK_DEVICES * 2];

#ifdef SADP
int	dk_cyl_index2[MAX_DISK_DEVICES] = {0};
int	*dk_cyl2[MAX_DISK_DEVICES];
#endif

/* Measurement hooks for disc3.c */
int	dk_ndrive3 = 0;
char	dk_busy3[MAX_DISK_DEVICES] = {0};
long	dk_time3[MAX_DISK_DEVICES];
long	dk_seek3[MAX_DISK_DEVICES];
long	dk_xfer3[MAX_DISK_DEVICES];
long	dk_wds3[MAX_DISK_DEVICES];
float	dk_mspw3[MAX_DISK_DEVICES];
long	dk_time_snap3[MAX_DISK_DEVICES];
struct timeval dk_resp3[MAX_DISK_DEVICES];

#ifdef SADP
int	dk_cyl_index3[MAX_DISK_DEVICES] = {0};
int	*dk_cyl3[MAX_DISK_DEVICES];
#endif


#ifdef DIAG0
extern	int	diag0();
int		(*io_diag)()	=	diag0;
#else
int		(*io_diag)()	=	0;
#endif /* DIAG0 */

/*
 * Streams subsystem
 */

#ifdef HPSTREAMS

int strmsgsz = STRMSGSZ;
int strctlsz = STRCTLSZ;
int nstrevent = NSTREVENT;
int nstrpush  = NSTRPUSH;

#include "../streams/str_hpux.h"
#include "../streams/str_stream.h"

#endif /* HPSTREAMS */


#ifdef NSDIAG0
/* #include	"../h/ns_diag.h" */
#include	"../sio/nsdiag0.h"
int		nsdiag0_high_water = 32;
nsdiag_event_msg_type	*nsdiag0_msg_queue;
#else /* NO_NSDIAG0 */
int		(*io_nsdiag)() = 0;
int		nsdiag0_high_water = 0;
#endif /* NSDIAG0 */

#include	"../sio/mux0.h"
#undef	reg
struct	cam_data_struct		* cam_data;
struct	mux_data_struct		* mux_data;
struct	tty			* mux0_ttys;
struct	mux_hwdata		* mux0_hwdata;
	int			  max_mux;

#undef	CMD_RESET
#undef	CMD_CLEAR
#include	"../sio/mux2.h"
struct	mux2_data_struct	* mux2_data;
struct	mux2_hwdata		* mux2_hwdata;
struct	tty			* mux2_ttys;
	int			  max_mux2;

#include "../sio/disc0.h"
#ifdef	NUM_DISC0
/* I/O buffers */
char	d0dbuf[sizeof(union genbuf) * (NUM_DISC0_L1+NUM_DISC0) + CPU_IOLINE-1];
char	d0rqblk[sizeof(union rqblk) * (NUM_DISC0_L1+NUM_DISC0) + CPU_IOLINE-1];
char	d0rpblk[sizeof(struct rpblk) * (NUM_DISC0_L1+NUM_DISC0) + CPU_IOLINE-1];
char	d0drqblk[sizeof(union drqblk) * (NUM_DISC0_L1+NUM_DISC0) + CPU_IOLINE-1];
/* Data structures */
struct dcs	d0dcs[NUM_DISC0_L1+NUM_DISC0];	  /* dev control struct */
cio_vquad_type	d0vq[(NUM_DISC0_L1+NUM_DISC0)*4]; /* CIO vquads		*/
struct d0msg	d0msg[NUM_DISC0_L1+NUM_DISC0];	  /* message templates	*/
struct buf	d0rq1[NUM_DISC0_L1];	/* I/O request list head	*/
struct buf	d0rq2[NUM_DISC0_L1];	/* I/O request list head	*/
struct buf	d0rbuf[NUM_DISC0_L1];	/* raw I/O request buf		*/
struct buf	d0ibuf[NUM_DISC0_L1];	/* ioctl I/O request buf	*/
idy_struct	d0idy[NUM_DISC0];	/* identify data		*/
int		d0model[NUM_DISC0_L1];	/* used to save model number	*/
int		num_disc0 = NUM_DISC0;
int		num_disc0_l1 = NUM_DISC0_L1;
#else
/* I/O buffers */
char	d0dbuf[1];
char	d0rqblk[1];
char	d0rpblk[1];
char	d0drqblk[1];
/* Data structures */
struct dcs	d0dcs[1];	  /* dev control struct */
cio_vquad_type	d0vq[1]; /* CIO vquads	       */
struct d0msg	d0msg[1];	  /* message templates	*/
struct buf	d0rq1[1];	/* I/O request list head	*/
struct buf	d0rq2[1];	/* I/O request list head	*/
struct buf	d0rbuf[1];	/* raw I/O request buf		*/
struct buf	d0ibuf[1];	/* ioctl I/O request buf	*/
idy_struct	d0idy[1];	/* identify data		*/
int		d0model[1];	/* used to save model number	*/
int		num_disc0 = 0;
int		num_disc0_l1 = 0;
#endif /* NUM_DISC0 */

#ifdef NUM_DIAGHPIB1
#include "../sio/diaghpib1.h"
#include "../sio/diagihpib1.h"
struct diaghpib1_pda *diaghpib1_pda_table[NUM_DIAGHPIB1];
int		num_diaghpib1 = NUM_DIAGHPIB1;
#else
int *diaghpib1_pda_table;
int		num_diaghpib1 = 0;
#endif /* NUM_DIAGHPIB1 */

#include "../sio/instr0_spc.h"
#ifdef NUM_INSTR0
struct hpib_pda *hpib_pda[NUM_INSTR0];
int		num_instr0 = NUM_INSTR0;
#else
struct hpib_pda hpib_pda[1];
int		num_instr0 = 0;
#endif /* NUM_INSTR0 */

#define MAX_LPR0 16
#if defined(NUM_LPR0) && (NUM_LPR0 > MAX_LPR0)
#    undef	MAX_LPR0
#    define	MAX_LPR0 NUM_LPR0
#endif	/*NUM_LPR0*/

#include "../h/lprio.h"
#include "../sio/lpr0_spc.h"
struct lp_pda	*lp_pda[MAX_LPR0];
int		max_lp = MAX_LPR0;

#undef dm_reply

#include "../sio/gpio0_spc.h"
#ifdef NUM_GPIO0
gpio0_pda_type *gpio0_pda_map[NUM_GPIO0];
int num_gpio0 = NUM_GPIO0;
#else
gpio0_pda_type *gpio0_pda_map[1];
int num_gpio0 = 0;
#endif /* NUM_GPIO0 */

#ifdef SIXR 
#include "../machine/sna_space.h"
#endif /* SIXR */

/*
 * lancift pointers for 10 interfaces
 */
int	lan_if_unit = -1;
#if (defined(LAN) || (defined(__hp9000s700) && defined(LAN01)))
/*
 * head of linked list of HP extended SAP/protocol mapping structures
 */
#include "../sio/lanc.h"

struct logged_link * log_hash_ptr[1<<HASHBITS];

lan_ift *lan_cio_ift_ptr[10];
lan_ift *lan_nio_ift_ptr[10];
#endif


#ifdef X25
#if (defined(NUM_PDN0) && (NUM_PDN0 >= 0))

#ifndef IPPROTO_ICMP
#include "../h/protosw.h"
#include "../netinet/in.h"
#endif /* NOT IPPROTO_ICMP */

#ifndef IFF_UP
#include "../net/if.h"
#endif /* IFF_UP not defined */

#include "../x25/x25gen.h"

#endif /* NUM_PDN0 */
#endif /* X25 */

/* NetWare MUST be after X25 in space.h. */
#ifdef NETWARE
#include "../netware/nwspace.h"
#endif /* NETWARE */

/* Space configuration stuff for Series 800 Graphics */
#ifndef NUM_GRAPH0
#   define NUM_GRAPH0 0
#endif
#ifndef NUM_GRAPH2
#   define NUM_GRAPH2 0
#endif
#ifndef NUM_DISPLAY0
#   define NUM_DISPLAY0 0
#endif

#ifdef KTEST
    /* used by ktest for automated 8042-based HIL testing */
    int hil_proc(), (*hil_proc_ptr)()=hil_proc;
#else
    (*hil_proc_ptr)() = NULL;
#endif

#ifdef _WSIO
#   include "../graf.800/ite1_spc.h"
#else
#   include "../graf.800/kbd0_spc.h"
#   include "../graf.800/hil0_spc.h"
#   include "../graf.800/ite0_spc.h"
#   include "../graf.800/graph0_spc.h"
#   include "../graf.800/graph2_spc.h"
#   include "../graf.800/displ_spc.h"
#endif
#include "../graf.800/frameb_spc.h"

/* Configurable constants */
/* a -1 means calculate the parameter at boot time */
#define VHNDFRAC	-1
#define MAXPMEM		-1

#include "../h/map.h"
#include "../h/sar.h"
struct map *sysmap;

#include "../h/sysinfo.h"
struct sysinfo sysinfo;
struct syswait syswait;
struct syserr syserr;

#include "../h/var.h"
struct var v = {
#ifdef	LATER
	NBUF,
	NCALL,
	NINODE,
	(char *)(&inode[NINODE]),
	NFILE,
	(char *)(&file[NFILE]),
	NMOUNT,
	(char *)(&mount[NMOUNT]),
	NPROC,
	(char *)(&proc[1]),
	0,
	(char *)0,
#endif	/* LATER */
#ifdef	LATER
	NCLIST,
	NSABUF,
	MAXUP,
	0,
	NHBUF,
	NHBUF-1,
	NPBUF,
	0,
	0,
#endif	/* LATER */
	VHNDFRAC,
#ifdef	LATER
	MAXPMEM,
	MAXDBUF
#endif	/* LATER */
};

#include "../h/pfdat.h"
struct pfdat	phead;		/* Head of free page list. */
struct pfdat	*pfdat;		/* Page frame database. */
struct pfdat	**phash;	/* Page hash access to pfdat. */
struct pfdat	ptfree;		/* Head of page table free list. */
int		phashmask;	/* Page hash mask. */

#include "../h/swap.h"
#include "../h/sema.h"

/*
 * used by the swapping subsystem
 */
/*
 * maximum number of swapdevics
 */
#ifndef	      NSWAPDEV
#define NSWAPDEV 10
#endif
int swapmem_on = SWAPMEM_ON;
int nswapdev = NSWAPDEV;
#ifndef _WSIO	/* weed out the s700 */
struct swdevt swdevt[NSWAPDEV];
#endif	/* !_WSIO */

/*
 * maximum number of file systems that can be enabled for swap
 */
int nswapfs = NSWAPFS;
fswdev_t fswdevt[NSWAPFS];

struct swap_stats swap_stats[NSWAPDEV+NSWAPFS+1];

/*
 * size of each swaptab[] entry in DEV_BSIZE units
 */
int swchunk = SWCHUNK;

int maxswapchunks = MAXSWAPCHUNKS;
int minswapchunks = MINSWAPCHUNKS;

devpri_t	swdev_pri[NSWPRI];
fspri_t		swfs_pri[NSWPRI];

swpt_t swaptab[MAXSWAPCHUNKS];
int   sysmem_max = SYSMEMMAX;

vm_sema_t	swap_lock;
int		swapwant;


struct minfo minfo;

int	maxpmem = MAXPMEM;	/* Max physical memory to use.	Used for */
				/* system performance measurement purposes */

#include "../sio/disc1.h"
struct disc1_pda *disc1_pdas[MAX_NUM_CS80_DEVICES]; /* virtual pda address
						  to lu map */
struct disc1_pda *disc1_real_pdas[MAX_NUM_CS80_DEVICES]; /* real pda address */

/* Uxgen selects one of the following to optionally allow */
/* two-byte characters in file names. */
/* extern int pn_getcomponent_chinese_t(); not supported yet */
extern int pn_getcomponent_n_computer();
#ifndef PARSER
#define PARSER pn_getcomponent_n_computer
#endif
int (*pn_getcomponent)() = PARSER;

#ifdef DSKLESS
struct pidchunk
{
	int start;
	int end;
} mypidchunks[NPROC];
#endif /* DSKLESS */


int dmmin = DMMIN;
int dmmax = DMMAX;
int dmshm = DMSHM;
int dmtext = DMTEXT;

#if defined(MIRROR)
#define disc2_open	mdisc2_open
#define disc2_close	mdisc2_close
#define disc2_size	mdisc2_size
#define disc2_strategy	mdisc2_strategy
#define disc2_read	mdisc2_read
#define disc2_write	mdisc2_write
#define disc2_ioctl	mdisc2_ioctl
#define disc2_dump	mdisc2_dump
#define disc2		mdisc2
#endif /* MIRROR */

#include "../cdfs/cdfsdir.h"
#include "../cdfs/cdnode.h"
#include "../cdfs/cdfs.h"

/*The NCDNODE should be defined in master for configurability. Before we
can actually do it, this is what we can do now.*/
#define NCDNODE 150
int	ncdnode = NCDNODE;

#ifdef BX25
#include "../machine/bx25.space.h"
#endif /* BX25 */

#ifdef NUM_DKITPPI0

#define DKCHANS		128
#define NUMREGSET	8
#define MAXDKIT		4	/* max number of interfaces */
#define MAXDKCHANS	128  /* NUMBER OF CHANS PER DKITPPI0 */
#define DKCBSIZE	256	/* the size of dkcblock */
#define MAXDKCHANSHIFT	  7  /* LOG2 of MAXDKCHANS */

/*
 * machine/space.h will include the following lines
 * should datakit become a standard product.
 *
 * #if DKIT
 * #include "../dkit/dkspace.h"
 * #endif
 *
 * but now the file is included in dkconf.c
 */

/*
 * The real dkit/dkspace.h
 */

#ifdef NUM_DKITPPI0
#define MAXDKCB		(NUM_DKITPPI0 * MAXDKCHANS * 2)
#else
#define MAXDKCB		2
#endif

int	maxdkcb = MAXDKCB;
int	maxdkchans = MAXDKCHANS;
int	dkcbsize = DKCBSIZE;
int	delayclosetimer = DELAYCLOSETIMER;

#include "../h/types.h"
#include "../sys/dkit.h"
#include "../sys/dkdev.h"
#include "../sys/dkcmc.h"
#include "../sys/dkkmc.h"
#include "../sys/dktrace.h"
#include "config.h"

char			dkcblock[(MAXDKCB+1)*DKCBSIZE];
caddr_t			dkcblockstack[MAXDKCB];

int	trace_wanted = 0;	/* a flag to indicate /dev/ppitrace is opened */

int	 ppi_created = 0;/* number of PPI cards recognized and
				   configured by CONFIGURATOR.		*/
struct buf	*remove_aev();

/* dk_hex digits used by printhex() to convert an integer number into hex */
char dk_hex[16] = {'0','1','2','3','4','5','6','7','8','9',
			'a','b','c','d','e','f'};

/* for debugging purpose */
int	dk_numread = 0;
int	numaev;

int	dktrap;
int	  chanmask = 0;
/* struct    ppi0_msg	msgq[NUMREGSET]; ccc 032889  for 7.0 */
int	dkdebug = 512;		/* not used - here for dk.c extern */

/*  int dkactive[MAXDKIT] = {0,0,0,0};	 */
int	dkactive[MAXDKIT] = {-1,-1,-1,-1};
int	vcsisup[MAXDKIT] = { 0, 0, 0, 0};


int	dkpanic = 0;		/* # of dk_close(0)'s in this run */

short dup_count = 0; /* counter for number of duplicate sends */

#ifdef NUM_DKITPPI0
int			dk_nchan =  NUM_DKITPPI0 * MAXDKCHANS;
int			dk_ccnt =   NUM_DKITPPI0;
char			cmcinpbuf [(NUM_DKITPPI0+1)*DKCBSIZE];
char			*cmcinp	   [NUM_DKITPPI0];
short			dkcactive  [NUM_DKITPPI0];
short			dkc1omplete[NUM_DKITPPI0] ;
short			dkc2omplete[NUM_DKITPPI0] ;
short			dkcstall[ NUM_DKITPPI0] ;
int			dk_achan[ NUM_DKITPPI0];
int			dkserverid[ NUM_DKITPPI0 ];
struct dkdev		dkdev	[ NUM_DKITPPI0 * MAXDKCHANS ];
struct tty		dkt_ttys[ NUM_DKITPPI0 * MAXDKCHANS ];
struct dksetupreq	dkreq	[ NUM_DKITPPI0 * MAXDKCHANS ];
struct	  dkchan	dkit	[ NUM_DKITPPI0 * MAXDKCHANS ];
struct dktracebuf	dktracebuf[ NUM_DKITPPI0 * MAXDKCHANS ];
#else
int			dk_nchan = 0;
int			dk_ccnt = 0;
char			cmcinpbuf[2*DKCBSIZE];
char			*cmcinp[1];
short			dkcactive[1];
short			dkc1omplete[1] ;
short			dkc2omplete[1] ;
short			dkcstall[1] ;
int			dk_achan[1];
int			dkserverid[1];
struct dkdev		dkdev[1];
struct dksetupreq	dkreq[1];
struct dkchan		dkit[1];
struct dktracebuf	dktracebuf[1];
#endif /* NUM_DKITPPI0 */

#ifdef DKRAW
#else
#define dkr_open	nodev
#define dkr_close	nodev
#define dkr_read	nodev
#define dkr_write	nodev
#define dkr_ioctl	nodev
#endif /* DKRAW */

#ifdef DKTTY
#	ifdef NUM_DKITPPI0
/* struct tty	dkt_ttys[ NUM_DKITPPI0 * MAXDKCHANS ];	*/
#	else
struct tty	dkt_ttys[ 1 ];
#	endif
#else
#define dkt_open	nodev
#define dkt_close	nodev
#define dkt_read	nodev
#define dkt_write	nodev
#define dkt_ioctl	nodev
#define dkt_select	nodev
#define dkt_option1	nodev
#define dkt_control	nodev
#endif /* DKTTY */

#ifdef DKXQT
#else
#define dkx_open	nodev
#define dkx_close	nodev
#define dkx_read	nodev
#define dkx_write	nodev
#define dkx_ioctl	nodev
#define dkx_select	nodev
#endif /* DKXQT */

/* We have modified llconf.o in order to have the kernel recognize
 * the ppi card.  However, DATAKIT code does not reference anything
 * from llconf.o.  The uxgen process, using 'ld ... libdkit.a libhp-ux.a...',
 * grabs llconf.o from libhp-ux.a since that archive is open when
 * it is first referenced.
 * The following unused pointer to a function simply causes the linker to
 * resolve *something* from llconf.o while scanning libdkit.a.	If
 * this is not done, llconf.o will be taken from libhp-ux.a during
 * customer gen's.
 */

extern int	ioconf();
int	(*ldfudge)() = ioconf;

/* ppi space declaration */

#ifdef NUM_DKITPPI0
#include "../dkit/nioppi.h"
#include "../dkit/ppi0.h"
#include "../dkit/ppi0_spc.h"

 struct ppi0_pda      *ppi_pda[NUM_DKITPPI0];
 int		      num_ppi0 = NUM_DKITPPI0;
 int		      dk_cnt   = 7;	  /* 128 channel per interface	*/
 struct ppi0_step     ppi_steps[NUM_DKITPPI0][NUMSTEPS+1];


/* special debug/printf flag */
int	dkitdebug;
int	cfreemap[((100 + 16 * MAXUSERS) >> 5) + 1];

#endif /* NUM_DKITPPI0 */
#endif

#if (defined(NUM_DKHS) && (NUM_DKHS > 0))
#include "../dkit/dkspace.h"
#endif /* NUM_DKHS */

int label_configured = 0;

/* File system async flag. If set file system data structures
   are written asychronously.		      */

int fs_async = FS_ASYNC;

/*
 * flag to control creation of "fast" symbolic links.
 */
int create_fastlinks = CREATE_FASTLINKS;

/* 
 * flag to turn off new AES conformance behavior for hp-ux system calls.
 */
int hpux_aes_override = AES_OVERRIDE;

/* hash table size scale with number of items hashed */

/* lpow2 returns largest power of 2 less than arg, min value 16, max 8192 */
#define lpow2(arg) \
	(arg) < 32? 16: \
	(arg) < 64? 32: \
	(arg) < 128? 64: \
	(arg) < 256? 128: \
	(arg) < 512? 256: \
	(arg) < 1024? 512: \
	(arg) < 2048? 1024: \
	(arg) < 4096? 2048: \
	(arg) < 8192? 4096: \
	8192

#ifdef SCALE_PERF_TEST
#define hashsize(length, items, default) default
#else
#define hashsize(length, items, default) \
	(lpow2((items)/(length)))
#endif

/* proc table */

#define PIDHSZ hashsize(4, NPROC, 64)
int	PIDHMASK = PIDHSZ - 1;
short pidhash[PIDHSZ];

#define PGRPHSZ hashsize(4, NPROC, 64)
int	PGRPHMASK = PGRPHSZ - 1;
short pgrphash[PGRPHSZ];

#define UIDHSZ hashsize(4, NPROC, 64)
int	UIDHMASK = UIDHSZ - 1;
short uidhash[UIDHSZ];

#define SIDHSZ hashsize(4, NPROC, 64)
int	SIDHMASK = SIDHSZ - 1;
short sidhash[SIDHSZ];


/* sleep table */

#define SQSIZEDEF hashsize(4, NPROC, 64)
int	SQSIZE	= SQSIZEDEF;
int	SQMASK	= SQSIZEDEF-1;

sema_t slpsem[SQSIZEDEF];

/* buffer table */

/* average buf hash chain length desired -- see machdep.c */
int bufhash_chain_length = 4;
struct	bufhd *bufhash; /* buffer hash table */
int	BUFHSZ, BUFMASK;/* size and mask for accessing bufhash */

/* inode table */

#define INOHSZDEF hashsize(6, NINODE, 64)
int	INOHSZ	= INOHSZDEF;
int	INOMASK = INOHSZDEF-1;
union ihead {				/* inode LRU cache, Chris Maltby */
	union  ihead *ih_head[2];
	struct inode *ih_chain[2];
} ihead[INOHSZDEF];


/* spinlocks */

#define SPINSIZEDEF (B_SEMA_HTBL_SIZE + SQSIZEDEF + 50)
int MAX_SPINLOCKS = SPINSIZEDEF;

/* Align on maximum cache line size */
#pragma align	64

lock_t	spin_alloc_base[SPINSIZEDEF] = { 0 };
lock_t	*spin_alloc_end = spin_alloc_base + SPINSIZEDEF;

extern nop();
int (*eisa_init_routine)() = nop;
int (*eisa_nmi_routine)() = nop;
int (*eisa_eoi_routine)() = nop;

#if defined(_WSIO) && defined(__hp9000s700)
unsigned int eisa_io_estimate = EISA_IO_ESTIMATE;
unsigned int vme_io_estimate  = VME_IO_ESTIMATE;
#endif /* s700 only */

/* DIL I/O and other buffers */
#include "../machine/iobuf.h"
#include "../machine/dilio.h"
int 	ndilbuffers = NDILBUFFERS;
struct	buf	dil_bufs[NDILBUFFERS];
struct	iobuf	dil_iobufs[NDILBUFFERS];
struct	dil_info dil_info[NDILBUFFERS];
int (*fhs_timeout_proc)() = NULL;


/* The following structure is not used or referenced by any other piece
 * of the kernel.  Its purpose is to allow an engineer debugging a
 * kernel to determine the sizes of certain key kernel structures.
 * This is useful when the exact sources are not available, as a sanity
 * check to determine which options were used in compilation, and as
 * a quick reference to determine the address of an element of a
 * table, given its index.  We'll start with certain key structures;
 * if others seem useful, just add them to the list.  If it's defined
 * in an ifdef, be sure to include the ifdef in the table.
 */

struct struct_size_struct {
	char *name;
	int   size;
};

#include "../h/spinlock.h"
#include "../h/sem_alpha.h"
#include "../h/sem_beta.h"
#include "../h/sem_sync.h"
#include "../h/region.h"
#include "../h/pregion.h"
#include "../h/vas.h"

struct struct_size_struct structure_sizes [] = {
	"proc",	      sizeof (struct proc),
	"user",	      sizeof (struct user),
	"buf",	      sizeof (struct buf),
	"tty",	      sizeof (struct tty),
	"pty_info",   sizeof (struct pty_info),
	"pcb",	      sizeof (struct pcb),
	"rusage",     sizeof (struct rusage),
	"ucred",      sizeof (struct ucred),
	"lock",	      sizeof (struct lock),
	"alpha sema", sizeof (struct sema),
	"beta sema",  sizeof (struct b_sema),
	"sync sema",  sizeof (struct sync),
	"pfdat",      sizeof (struct pfdat),
	"region",     sizeof (struct region),
	"pregion",    sizeof (struct pregion),
	"vas",	      sizeof (struct vas),
	0, 0
};

#endif /* _KERNEL */

#endif /* ! _MACHINE_SPACE_INCLUDED */
