/*
 * @(#)spl.h: $Revision: 1.35.61.6 $ $Date: 93/02/24 12:32:48 $
 * $Locker:  $
 * 
 */

#ifndef	SPL_INCLUDED
#define	SPL_INCLUDED

/*
 * This files defines the processor (spl()) levels. It also assigns certain
 * eirr bits. For the series 800, the following interrupt priority levels
 * are defined:
 *
 *
 *	SPLPREEMPTOK() - allow all interrupts
 *		Enables LPMC, powerfail and all eirr bits, except
 *		HPMC_INT, which is reserved for crash dump use.
 *
 *	SPLNOPREEMPT() - disable kernel preemption
 *		Same as SPLPREEMPTOK(), except 'trap type' interrupts are
 *		disabled. 'trap type' interrupts cause the kernel to be
 *		preempted by another process when at SPLPREEMPTOK().
 *		SPLNOPREEMPT() and higher spl levels indicate the kernel
 *		cannot be preempted.
 *		Currently there are 2 'trap type' interrupts:
 *			MPSCHED -  In an MP system, one cpu tells another cpu
 *			  	   to check the runq by sending this interupt
 *				   type.
 *			KPREEMPT - When the kernel is executing and a higher
 *				   priority process should run (eg. clock tick
 *				   recomputed priorities or external interrupt
 *				   caused a wakeup()), the kernel sends itself
 *				   a KPREEMPT interrupt to cause a preemption
 *				   when the spl level is SPLPREEMPTOK().
 *
 *	SPL2() - disable software interrupts
 *		Same as SPLNOPREEMPT(), except 'software priority' interrupts
 *		are disabled. Software interrupts are often sent when
 *		the kernel is executing on the ics at a high interrupt priority
 *		and it wants to schedule some processing to be done later after
 *		all high priority interrupts (eg. device IO) are serviced.
 *		The following software interrupts are used:
 *			SOFTSTREAM - not sure how its used
 *			SOFTNET    - When a networking IO driver receives an
 *				     inbound packet, it removes the packet
 *				     and schedules other networking code to
 *				     process the packet at a lower priority.
 *				     by sending a SOFTNET interrupt.
 *			SOFTCLOCK  - When a clock tick occurs, hard clock
 *				     updates the time and does a little work
 *				     and schedules the potentially lengthy
 *				     work (such as handling callouts)
 *				     by sending a SOFTCLOCK interrupt.
 *		In addition to software interrupts, SPL2() also disables
 *		interrupts from low priority Type-B DMA IO modules. Currently,
 *	        there is only one low priority IO module (ieee_8024 - map).
 *
 *	SPL5() - disable IO modules
 *		 Same as SPL2(), except all IO module interrupts are disabled.
 *
 *	SPL6() - disable hard clock and kernel powerfail recovery
 *		 Same as SPL5() except hard clock interrupts are disabled.
 *
 *	SPL7() - disable the world
 *		 Same as SPL6(), except powerfail and RDB interrupts and LPMC
 *		 are disabled - PSW_I is off.
 */

/*
 * WARNING: many of these defines are used by assembly. Therefore,
 * 'C' operators ahould not be used (eg. & | >>, (), etc.)
 */


#ifndef _SYS_STDSYMS_INCLUDED
#ifdef _KERNEL_BUILD
#    include "../h/stdsyms.h"
#else  /* ! _KERNEL_BUILD */
#    include <sys/stdsyms.h>
#endif /* _KERNEL_BUILD */
#endif  /*  _SYS_STDSYMS_INCLUDED  */

#ifdef _KERNEL

#define MSB	0x80000000

#define CLOCK_INT_BIT		0
#define CLOCK_INT_ENABLE		0x80000000

#define REMOTE_PFAIL_INT_BIT	1
#define REMOTE_PFAIL_INT_ENABLE		0x40000000

#define KERN_PFR_INT_BIT	2
#define KERN_PFR_INT_ENABLE		0x20000000

#ifdef _WSIO
/*
*	WARNING: The EIRR bit currently used for the VSC interrupt line 
*		 (INTCPU.L) is bit 3 which is the same bit used for NIO
*		 Type-A Direct modules.  Current S700 systems do not support
*		 NIO so this is not viewed as a problem, but if NIO is
*		 ever supported this will need to be reviewed.
*
*	IO_INT_BIT - Used by WAX and the second LASI on GECKO.
*/
#define VSC_INT_BIT     	3
#define VSC_INT_ENABLE                  0x10000000
#define IO_INT_BIT		7
#define IO_INT_ENABLE			0x01000000
#define SW_TRIGGER_6_INT_BIT 	4
#define SW_TRIGGER_6_INT_ENABLE         0x08000000
#define VME_INT_BIT		8
#define VME_INT_ENABLE			0x00800000
#else
#define TYPE_A_DIRECT_INT_BIT	3
#define TYPE_A_DIRECT_INT_ENABLE	0x10000000
#endif  /* _WSIO */
/*
 * RDB uses bit 5 and 6
 */
#define	RDB_INT_ENABLE			0x06000000

/*
 *	First software priority interrupt bit.
 */
#ifdef _WSIO  /* this is an s800 file only, so this means HP-PA workstations */
#define SW_TRIGGER_5_INT_BIT    20
#define SW_TRIGGER_5_INT_ENABLE         0x00000800

#define SW_TRIGGER_4_INT_BIT    21
#define SW_TRIGGER_4_INT_ENABLE         0x00000400

#define SW_TRIGGER_3_INT_BIT    22
#define SW_TRIGGER_3_INT_ENABLE         0x00000200

#define SW_TRIGGER_2_INT_BIT    26
#define SW_TRIGGER_2_INT_ENABLE         0x00000020

#define SW_TRIGGER_1_INT_BIT    27
#define SW_TRIGGER_1_INT_ENABLE         0x00000010

#else

#define TYPE_B_DMA_LOW_PRI_INT_BIT	24
#define TYPE_B_DMA_LOW_PRI_INT_ENABLE	0x00000080

#endif 


/* The following #defines are defined in this way so that the commands folks
   can use the correct value at run-time.  Any variables that are added to
   this file and whose name is shared between WS and non-WS but whose value
   is different between them should be structured in the same way.  */


#define _WSIO_SOFTSTREAM_INT_BIT	23
#define _WSIO_SOFTSTREAM_INT_ENABLE     0x00000100

#define _WSIO_SOFTNET_INT_BIT		24
#define _WSIO_SOFTNET_INT_ENABLE        0x00000080

#define _WSIO_SOFTCLOCK_INT_BIT		25
#define _WSIO_SOFTCLOCK_INT_ENABLE      0x00000040

/**  workstation HP-PA vs other HP-PA  ***/

#define _SIO_SOFTSTREAM_INT_BIT		25
#define _SIO_SOFTSTREAM_INT_ENABLE	0x00000040

#define _SIO_SOFTNET_INT_BIT		26
#define _SIO_SOFTNET_INT_ENABLE		0x00000020

#define _SIO_SOFTCLOCK_INT_BIT		27
#define _SIO_SOFTCLOCK_INT_ENABLE	0x00000010

/* this file is s800 only so we don't need the #ifdef __hp9000s800  */
#ifdef _WSIO

#define SOFTSTREAM_INT_BIT		_WSIO_SOFTSTREAM_INT_BIT
#define SOFTSTREAM_INT_ENABLE		_WSIO_SOFTSTREAM_INT_ENABLE

#define SOFTNET_INT_BIT			_WSIO_SOFTNET_INT_BIT
#define SOFTNET_INT_ENABLE		_WSIO_SOFTNET_INT_ENABLE

#define SOFTCLOCK_INT_BIT		_WSIO_SOFTCLOCK_INT_BIT
#define SOFTCLOCK_INT_ENABLE		_WSIO_SOFTCLOCK_INT_ENABLE

#else

#define SOFTSTREAM_INT_BIT		_SIO_SOFTSTREAM_INT_BIT
#define SOFTSTREAM_INT_ENABLE		_SIO_SOFTSTREAM_INT_ENABLE

#define SOFTNET_INT_BIT			_SIO_SOFTNET_INT_BIT
#define SOFTNET_INT_ENABLE		_SIO_SOFTNET_INT_ENABLE

#define SOFTCLOCK_INT_BIT		_SIO_SOFTCLOCK_INT_BIT
#define SOFTCLOCK_INT_ENABLE		_SIO_SOFTCLOCK_INT_ENABLE

#endif  /* _WSIO */



/*
 *	First trap type interrupt bit.
 */

#define MPSCHED_INT_BIT		28
#define MPSCHED_INT_ENABLE		0x00000008

#define UNUSED_TRAP_INT_BIT29	29
#define UNUSED_TRAP_INT_ENABLE29	0x00000004

#define KPREEMPT_INT_BIT	30
#define KPREEMPT_INT_ENABLE		0x00000002
/*
 * HPMC_INT_BIT should never be enabled, except by crash dump assembly.
 */
#define HPMC_INT_BIT		31
#define HPMC_INT_ENABLE			0x00000001

/*
 * The value for SPL7 is arbitrary, since psw-I is off; it is less than
 * five bits so that comib can be used to test it.
 */

#define SPL7		0x00000007
#define SPL6		0x46000000

#ifdef _WSIO
#define SPL4            0xfffff800
#define SPL3            0xfffffc00
#define SPL1            0xffffffe0
#define SPL0            SPLNOPREEMPT
#endif  /* _WSIO */

#define _WSIO_SPL5            0xff800000
#define _WSIO_SPL2            0xfffffe00
#define _SIO_SPL5		0xe6000000
#define _SIO_SPL2		0xffffff00

/* this file is s800 only, so we don't need the #ifdef __hp9000s800 */
/* the following is structured this way for the commands folks.  So that they
   can have one executable for WS and non-WS HP-PA.  They determine the 
   correct value at run-time */
#ifdef _WSIO
#define SPL5		_WSIO_SPL5
#define SPL2		_WSIO_SPL2
#else
#define SPL5		_SIO_SPL5
#define SPL2		_SIO_SPL2
#endif


#define SPLNOPREEMPT    0xfffffff0
#define SPLPREEMPTOK	0xfffffffe

#define SPL6NORDB 	REMOTE_PFAIL_INT_ENABLE    /* See pfail.c */
#define SPL6NORPF       0x06000000
#define SPLIMP		SPL5
#define SPLNET		SPL2

#define LPRIV_INT_BIT		32	/* This is a fake interrupt bit used
					 * in take_a_trap() to cause a
					 * lower priv transfer trap.
					 */

#define MPLOCAL_MASK	(MPSCHED_INT_ENABLE	| KPREEMPT_INT_ENABLE	| \
			 SOFTNET_INT_ENABLE     | SOFTSTREAM_INT_ENABLE | \
			 SOFTCLOCK_INT_ENABLE	| RDB_INT_ENABLE	| \
			 CLOCK_INT_ENABLE	| REMOTE_PFAIL_INT_ENABLE |\
                         KERN_PFR_INT_ENABLE)
/*
 * BASEPRI is true when the spl level of eiem passed to us equals that
 *         used in the idle loop in swtch().
 */
#define	BASEPRI(eiem)	(eiem == (unsigned) SPLNOPREEMPT)

#endif /* _KERNEL */

#endif /* SPL_INCLUDED */
