/* @(#) $Revision: 1.2.109.10 $ */
#ifndef _NTL_INCLUDED
#define _NTL_INCLUDED


/* this file defines the constants used by the trace/log facilities */
/* version A.00.00                                                  */


#define   TL_TRACE_DEF_BUFSIZE    32*1024    /* default trace buf size */
#define   TL_LOG_DEF_BUFSIZE      8*1024     /* default log buf size */
#define   TL_MAX_BUFSIZE          512        /* max buf size in K bytes */
#define   TL_MAX_PORT_CT          8          /* max # of trace/log ports */
#define   TL_MAX_SUBSYS           256        /* Mixed ids for host + NS */



/* TL error return status related definitions */

/*
 * TL error return status is composed of 3 parts:
 *    -------------------------------------------------------------------------
 *    | error_type | TL assigned error code | type specific error code if any |
 *    |  (4 bits)  |       (12 bits)        |            (16 bits)            |
 *    -------------------------------------------------------------------------
 */

/*
 * TL error types:
 *    for TLERRTYPE_TL, type specific error code section would not be used;
 *    for TLERRTYPE_KTL , type specific error
 *        code is returned from respective modules;
 *    for UNIX system call, type specific error code is lower 16 bits of errno.
 */
#define   TLERRTYPE_TL            0x00000000 /* TL internal error */
#define   TLERRTYPE_KTL           0x10000000 /* KTL library call error */
#define   TLERRTYPE_SYS           0x40000000 /* UNIX system call error */

/* Masks for interpreting TL return error code */
#define   TLMASK_ERRORTYPE        0xF0000000 /* error type */
#define   TLMASK_TLERRCODE        0x0FFF0000 /* TL assigned error code */
#define   TLMASK_TYPESPERR        0x0000FFFF /* type specific error code */


/* TL assigned error codes for TL internal error */
#define   NO_ERR                  0x00000000
#define   TLERR_NO_PORT           0x00010000 /* no more ports available */
#define   TLERR_INV_BUFSIZE       0x00020000 /* invalid bufsize in tl_create */
#define   TLERR_INV_SUBSYS        0x00030000 /* invalid subsystem id */
#define   TLERR_IDEN_SUBSYS       0x00040000 /* subsystem already been bound */
#define   TLERR_INSUF_CAP         0x00050000 /* user not have sufficient cap
                                              * to perform following operations: 
                                              * 1. delete the port bound by others 
                                              * 2. attach a port bound by others
                                              * 3. detach a port attached to by others
                                              * 4. control mask or data size over
                                              *    other user's subsystem/port       */
/* #define   TLERR_INV_PORT       6           the port number is out of range */
#define   TLERR_PORT_RANGE        0x00060000 /* port number out of range */
/* #define   TLERR_INV_BIND_REQ   7            port bound to by a non-privileged
                                             * user before the super user requests
                                             * exclusive subsystem access */
#define   TLERR_PORT_UNAVAIL      0x00080000 /* port attached by other reader */
#define   TLERR_NO_DATA           0x00090000 /* no data to read */
#define   TLERR_BUF_SMALL         0x000A0000 /* read buffer is too small */
#define   TLERR_INV_DATA_SIZE     0x000B0000 /* set data size beyond bufsize */
#define   TLERR_INV_CTL_REQ       0x000C0000 /* not supported tl_control req */
/* #define   TLERR_UNKNOWN_SUBSYS 13           not currently used */
/* #define   TLERR_UNKNOWN_PORT   14           the portnum changed during tl_write */
#define   TLERR_PORT_STATE        0x000E0000 /* port not in right state */
#define   TLERR_PORT_IN_USE       0x000F0000 /* port allocated by other user */
/* #define   TLERR_NO_MEM         16           no memory available for port alloc */
#define   TLERR_SUBSYS_NOT_BOUND  0x00110000 /* subsystem not been bound */
#define   TLERR_READ_STOP         0x00140000 /* to stop read operation */
#define   TLERR_DATA_TOO_LONG     0x00150000 /* data to be written too long */
/* #define   TLERR_SYS_ERROR      22           system call error */
#define   TLERR_INV_FILE          0x00170000 /* invalid trace/log output file */
#define   TLERR_IPC_ERROR         0x00180000 /* ulipc interface error */
#define   TLERR_READER_PGM        0x001A0000 /* invalid reader program */
#define   TLERR_SIG_CAUGHT        0x001B0000 /* a signal is caught */
#define   TLERR_TLDEM_START       0x001C0000 /* fail to start tl daemons */
#define   TLERR_READ_ABORT        0x001D0000 /* read aborted */
/* 4/4/89: add following four for tl_check() */
#define   TLERR_MISMATCH_CLASS    0x00300000 /* class not matched */
#define   TLERR_MISMATCH_KIND     0x00310000 /* kind not matched */
#define   TLERR_SUBSYS_NOT_ENABLED 0x00320000 /* subsys not enabled */
#define   TLERR_SUBSYS_NOT_EXCL   0x00330000 /* ss not enabled exclusively */

/* TL assigned error codes for KTL related error */
#define   TLERR_KTL_STARTDEM      0x01010000 /* fail to start ktl daemon */
#define   TLERR_KTL_EXISTDEM      0x01020000 /* fail to detect ktl daemon */

/* TL assigned error codes for UNIX system error */
#define   TLERR_SEMCTL_INIT       0x04010000 /* fail to init sem */
#define   TLERR_SEMGET_CREATE     0x04020000 /* fail to create sem */
#define   TLERR_SEMOP_PSEM        0x04030000 /* fail to P_Sem */
#define   TLERR_SEMOP_VSEM        0x04040000 /* fail to V_Sem */
#define   TLERR_SEMCTL_RMID       0x04050000 /* fail to remove sem */
#define   TLERR_SHMAT             0x04060000 /* fail to attach to shm */
#define   TLERR_SHMCTL_LOCK       0x04070000 /* fail to lock shm */
#define   TLERR_SHMCTL_RMID       0x04080000 /* fail to remove shm */
#define   TLERR_SHMDT             0x04090000 /* fail to detach from shm */
#define   TLERR_SHMGET_CREATE     0x040A0000 /* fail to create shm */
#define   TLERR_SHMGET_ID         0x040B0000 /* fail to get shm id */
#define   TLERR_FORK_DEM          0x040C0000 /* fail to fork TL daemon */
#define   TLERR_FTOK_CTL          0x040D0000 /* fail to ftok for ctl table */
#define   TLERR_STAT_DEV          0x040E0000 /* fail to get card dev status */
#define   TLERR_STAT_KTLDEV       0x040F0000 /* fail to get ktl dev status */
#define   TLERR_OPEN_KTLDEV       0x04100000 /* fail to open ktl device */
#define   TLERR_IOCTL_KTL_START   0x04110000 /* fail on ioctl start */
#define   TLERR_IOCTL_KTL_SET_MASK    0x04120000 /* fail on ioctl setmask */
#define   TLERR_IOCTL_KTL_RESET_MASK  0x04130000 /* fail on ioctl resetmask */
#define   TLERR_IOCTL_KTL_STOP    0x04140000 /* fail on ioctl stop */
#define   TLERR_KILL_DEM          0x04150000 /* fail to kill TL daemon */
#define   TLERR_SEMOP_PSEM_NOUNDO 0x04160000 /* fail to p_sem w/o undo */
#define   TLERR_SEMCTL_SETSEM     0x04170000 /* fail to set sem val */
#define   TLERR_IOCTL_X25         0x04180000 /* fail on ioctl for X25 */
#define   TLERR_KTL_DEV           0x04190000 /* open failed, no device */
#define   TLERR_KTL_DRV           0x041A0000 /* open failed, no diver */
/* Following ones are temperary and possibly to be removed or renamed later */
#define   TLERR_OPEN              0x04510000 /* fail to open file */
#define   TLERR_FOPEN             0x04520000 /* fail to fopen file */
#define   TLERR_CLOSE             0x04530000 /* fail to close file */
#define   TLERR_FCLOSE            0x04540000 /* fail to fclose file */
#define   TLERR_UNLINK            0x04550000 /* fail to unlink file */
#define   TLERR_FCNTL_DUP         0x04560000 /* fail to dup using fcntl */
#define   TLERR_MKTEMP            0x04570000 /* fail to mktemp */
#define   TLERR_FTOK              0x04580000 /* fail to ftok */
#define   TLERR_OPEN_TLOUT        0x04590000 /* fail to open reader out file */
#define   TLERR_OPEN_TLTMP        0x045A0000 /* fail to open reader tmp file */
#define   TLERR_EXECL_DAEMON      0x045B0000 /* fail to execl daemon pgm */
#define   TLERR_EXECL_READER      0x045C0000 /* fail to execl reader pgm */
#define   TLERR_SYS_ERROR         0x04600000 /* tmp: other system call error */
#define   TLERR_X25OPEN           0x04610000 /* x25 device failed to open */



#ifndef _UL_PER_PATH_INFO
#define _UL_PER_PATH_INFO

#define ULDEF_PER_PATH_INFO            -1       /* default per path val */
typedef int             ul_int_t;
struct ul_per_path_info{
                ul_int_t        uid;
                ul_int_t        pathid;
                ul_int_t        lpr_mgc;
                ul_int_t        rpr_mgc;
        } ;
#endif
#define MAX_TLBUF 200          /* max pieces of data that comprise the msg */


typedef struct {
                 unsigned  trace_log_bit        : 1;
                 unsigned  prefilter_bit        : 1;
                 unsigned  reserved             : 30;
}  ntlwrt_flag_type;

/* tlwrt_flag_type structure:                    */
/* trace_log_bit: set it if it is for log, reset this bit if it */
/* is for trace                                                 */
/* user_proc_bit: set it if the caller is running as a user     */
/* process, reset it it if the caller is running as the front   */
/* end processor or kernel                                      */
/* vector_buf_bit: set this flag if multiple buffers are used,  */
/* reset it if a single buffer is used.                         */
/* exclusive_subsys_bit: set it if the subsystem is used        */
/* exclusivley by the superuser, reset it otherwise.            */
/* NOTE: this structure is included for backwards compatibility */
/* only.  Do not use this in new code.                          */

typedef struct {
                 unsigned  trace_log_bit        : 1;
                 unsigned  user_proc_bit        : 1;
                 unsigned  vector_buf_bit       : 1;
                 unsigned  exclusive_subsys_bit : 1;
                 unsigned  reserved             : 28;
}  tlwrt_flag_type;

/* log_instance : is used to associate the error code across the*/
/* subsystems.                                                  */
/* invoke_id : identify a particular instance of a trace/log    */
/* entity (usually a process ID).                               */
/* cid : Connection Id.  For host subsystems this is the valid  */
/* connection identifier returned from CM.  For host subsystem  */
/* who don't yet have a connection or for card subsystems, this */
/* field should be set to -1.                                   */
/* uid : User Id.  This is the user ID of the calling process.  */
/* pathid : identifies a particualr connection/path on the host.*/
typedef struct {
                unsigned short	log_instance;
                int        	invoke_id;
		int		cid;
		int		uid;
		int		pathid;
} ntlwrt_info_type;

/* info_ct : the number of information fields contained in this */
/* structure.  It is set to 6.                                  */
/* log_instance : is used to associate the error code across the*/
/* subsystems.                                                  */
/* invoke_id : identify a particular instance of a trace/log    */
/* entity.                                                      */
/* cid : Connection Id.  For host subsystems this is the valid  */
/* connection identifier returned from CM.  For host subsystem  */
/* who don't yet have a connection or for card subsystems, this */
/* field should be set to -1.                                   */
/* ppi : Per path info.  A pointer to a struct ul_per_path_info */
/* gotten from a HPS call (OCR #116).  Struct ul_per_path_info  */
/* is defined in ul_ipc.h.                                      */
/* NOTE: this structure is included for backwards compatibility */
/* only.  Do not use this in new code.                          */
typedef struct {
                 short     info_ct;
                 short     log_instance;
                 int       invoke_id;
                 int       cid;
                 struct ul_per_path_info    *ppi;
} tlwrt_info_type;
                 
/* vector_buf_type definition:                   */
/* bufptr: pointer to a buffer containing the    */
/* trace/log data                                */
/* buflen: the size of the trace/log message in  */
/* bytes                                         */

typedef struct {
                 char            *bufptr;
                 int             buflen;
}  vector_buf_type;


/* tlwrt_buf_type definition:                    */
/* if the vector_buf_bit is not set, the pointer */
/* to a single buffer is used, the pointer to an */
/* array of structure vector_buf_type is used    */
/* otherwise                                     */

typedef union  {
                 char            *single_tlbuf;
                 vector_buf_type *vector_tlbuf;
}  tlwrt_buf_type;





/*
 * **********************************************************************
 *
 * The following are needed by subformatters.
 *
 * **********************************************************************
 */

#define NTL_VERSION	900	/* version id used for backwards  */
                                /* compatibility in the formatter */

#define CALLED_BY_NKTLD	0xfc	/* Arbitrary number used to signal */
                                /* ntl_write that it was called by */
                                /* nktl_daemon and the daemon has  */
                                /* encoded the ktl_msg_hdr as the  */
                                /* first element of the iovec.     */

#ifndef _TIME_H
#include  <time.h>
#endif

#ifndef UNDEF_SET_OF_32   /* for avoiding conflict with llio.h */
typedef   unsigned int  set_of_32;
#endif
typedef   int           err_num;
typedef   int           error_code;


/* tl_msg_hdr :                                                 */
/* each trace/log message is composed of 2 parts: header        */
/* information and data.  The header format is used by both the */
/* trace and log message and it is provided through the trace/  */
/* log facilities.                                              */


typedef struct {
                 unsigned              trace_log_bit : 1;
                 unsigned              user_proc_bit : 1;
                 unsigned              reserved      : 30;
}  tl_msg_flag_type;

typedef struct {
                 unsigned short        hdr_len;
                 short                 subsystemid;
                 int                   device_id;
                 tl_msg_flag_type      flags;  
                 set_of_32             kind;
                 set_of_32             class;
                 set_of_32             subclass;
                 unsigned int          dropped_events;
                 unsigned int          dropped_data;
                 unsigned int          data_len;
                 unsigned int          orig_data_len;
                 struct timeval        time;
                 int                   invoke_id;
                 int                   path_id;  
                 unsigned short        log_instance; 
                 unsigned short        uid;
                 unsigned int          connection_id;
}  tl_msg_hdr_type;

#endif
